// $Id: Application.cpp,v 1.19 2013/10/25 12:27:17 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Application.h"
#include "ConfigurationDialog.h"
#include "File.h"
#include "Icons.h"
#include "IconEngine.h"
#include "LogDialog.h"
#include "MainMenu.h"
#include "MessageQueue.h"
#include "SystemTrayIcon.h"
#include "Util.h"
#include "XmlOptions.h"
#include "CustomProcess.h"

#include <QApplication>
#include <QDesktopWidget>

//____________________________________________
void Application::usage( void )
{
    Debug::Throw(0) << "usage : qssh [options] [host]" << endl;
    SERVER::ApplicationManager::commandLineParser().usage();
    return;
}

//____________________________________________
Application::Application( CommandLineArguments arguments ):
    BaseApplication( 0, arguments ),
    Counter( "Application" ),
    systemTrayIcon_( 0 ),
    messageQueue_( 0 ),
    logDialog_( 0 )
{}

//____________________________________________
Application::~Application( void )
{

    Debug::Throw( "Application::~Application.\n" );
    if( systemTrayIcon_ ) delete systemTrayIcon_;
    if( messageQueue_ ) delete messageQueue_;
    if( logDialog_ ) delete logDialog_;

}

//____________________________________________
bool Application::realizeWidget( void )
{
    Debug::Throw( "Application::realizeWidget.\n" );

    // check if the method has already been called.
    if( !BaseApplication::realizeWidget() ) return false;

    // rename about action
    aboutAction().setText( tr( "About Qssh" ) );
    configurationAction().setText( tr( "Configure Qssh..." ) );

    // log
    toggleLogAction_ = new QAction( IconEngine::get( ICONS::FIND ), tr( "Show Log" ), this );
    toggleLogAction_->setCheckable( true );
    toggleLogAction_->setChecked( false );

    // top widget
    systemTrayIcon_ = new SystemTrayIcon();

    // message queue
    messageQueue_ = new MessageQueue();

    // log
    logDialog_ = new LogDialog();

    // connections
    connect( systemTrayIcon_, SIGNAL( messageAvailable( QString ) ), SLOT( _updateMessageQueue( void ) ) );
    connect( systemTrayIcon_, SIGNAL( messageAvailable( QString ) ), &messageQueue(), SLOT( processMessage( QString ) ) );
    connect( &systemTrayIcon_->mainMenu(), SIGNAL( logMessageAvailable( QString ) ), &logDialog(), SLOT( processMessage( QString ) ) );
    connect( toggleLogAction_, SIGNAL( toggled( bool ) ), &logDialog(), SLOT( setVisible( bool ) ) );

    // show main window
    emit configurationChanged();
    systemTrayIcon_->show();

    // timer is needed so that geometry is set properly at first connection
    // use 2 seconds delay. This is arbitrary.
    timer_.start( 2000, this );

    return true;

}

//___________________________________________________________
void Application::_configuration( void )
{
    Debug::Throw( "Application::_configuration.\n" );
    emit saveConfiguration();
    ConfigurationDialog dialog;
    connect( &dialog, SIGNAL( configurationChanged() ), SIGNAL( configurationChanged() ) );
    dialog.centerOnWidget( qApp->activeWindow() );
    dialog.exec();

}

//___________________________________________________________
void Application::_updateMessageQueue( void )
{

    // get desktop size
    QDesktopWidget *desktop( qApp->desktop() );
    QSize desktop_size( desktop->frameSize() );
    QRect geometry( systemTrayIcon_->geometry() );

    static const int offset = 10;
    if( geometry.bottom() > desktop_size.height()/2 )
    {

        messageQueue().setDirection( MessageQueue::Ascending );
        messageQueue().setVerticalOffset( geometry.top() - offset );

    } else {

        messageQueue().setDirection( MessageQueue::Descending );
        messageQueue().setVerticalOffset( geometry.bottom() + offset );

    }

}

//___________________________________________________________
void Application::_runKeychain( void )
{
    Debug::Throw( "Application::_runKeychain.\n" );

    // find key file
    File file;
    QTextStream( &file ) << Util::home() << ".ssh/id_rsa";
    if( !file.exists() )
    {
        Debug::Throw(0) << "Application::_runKeychain - could not find file" << file << endl;
        return;
    }

    // run process
    CustomProcess process;
    process.start( QStringList()
        << XmlOptions::get().raw( "KEYCHAIN_COMMAND" )
        << "/home/hpereira/.ssh/id_rsa" );
    if( !process.waitForStarted() )
    {
        Debug::Throw( "Application::_runKeychain - could not start process.\n" );
        return;
    }

    if( !process.waitForFinished() )
    {
        Debug::Throw( "Application::_runKeychain - process did not terminate properly.\n" );
        return;
    }
}

//___________________________________________________________
void Application::timerEvent( QTimerEvent* event )
{
    if( event->timerId() == timer_.timerId() )
    {
        timer_.stop();
        systemTrayIcon_->mainMenu().autoConnect();
    }
}
